package controller

import (
	"electricity_bill_calc/model"
	"electricity_bill_calc/repository"
	"electricity_bill_calc/response"
	"electricity_bill_calc/security"
	"electricity_bill_calc/service"
	"electricity_bill_calc/utils"
	"net/http"

	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
	"github.com/jinzhu/copier"
	"github.com/shopspring/decimal"
)

type _ParkController struct {
	Router *gin.Engine
}

var ParkController *_ParkController

func InitializeParkController(router *gin.Engine) {
	ParkController = &_ParkController{
		Router: router,
	}
	ParkController.Router.GET("/parks", security.EnterpriseAuthorize, listAllParksUnderSessionUser)
	ParkController.Router.GET("/parks/:uid", security.ManagementAuthorize, listAllParksUnderSpecificUser)
	ParkController.Router.POST("/park", security.EnterpriseAuthorize, createNewPark)
}

func listAllParksUnderSessionUser(c *gin.Context) {
	result := response.NewResult(c)
	session, exists := c.Get("session")
	if !exists {
		result.Error(http.StatusUnauthorized, "用户会话无效。")
		return
	}
	userSession, ok := session.(*model.Session)
	if !ok {
		result.Failure(http.StatusInternalServerError, "内部缓存错误，需要重新登录。")
		return
	}
	parks, err := repository.ParkRepo.ListAllParkBelongsTo(userSession.Uid)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	result.Json(http.StatusOK, "已获取到指定用户下的园区。", gin.H{"parks": parks})
}

func listAllParksUnderSpecificUser(c *gin.Context) {
	result := response.NewResult(c)
	requestUserId := c.Param("uid")
	parks, err := repository.ParkRepo.ListAllParkBelongsTo(requestUserId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	result.Json(http.StatusOK, "已获取到指定用户下的园区。", gin.H{"parks": parks})
}

type _ParkCreationFormData struct {
	Name     string              `json:"name" form:"name"`
	Region   *string             `json:"region" form:"region"`
	Address  *string             `json:"address" form:"address"`
	Contact  *string             `json:"contact" form:"contact"`
	Phone    *string             `json:"phone" from:"phone"`
	Area     decimal.NullDecimal `json:"area" from:"area"`
	Capacity decimal.NullDecimal `json:"capacity" from:"capacity"`
	Tenement decimal.NullDecimal `json:"tenement" from:"tenement"`
	Category int                 `json:"category" form:"category"`
	Submeter int                 `json:"submeter" form:"submeter"`
}

func createNewPark(c *gin.Context) {
	result := response.NewResult(c)
	session, exists := c.Get("session")
	if !exists {
		result.Error(http.StatusUnauthorized, "用户会话无效。")
		return
	}
	userSession, ok := session.(*model.Session)
	if !ok {
		result.Failure(http.StatusInternalServerError, "内部缓存错误，需要重新登录。")
		return
	}
	formData := new(_ParkCreationFormData)
	c.BindJSON(formData)
	newPark := new(model.Park)
	copier.Copy(newPark, formData)
	newPark.Id = uuid.New().String()
	newPark.UserId = userSession.Uid
	nameAbbr := utils.PinyinAbbr(newPark.Name)
	newPark.Abbr = &nameAbbr
	newPark.Enabled = true
	err := service.ParkService.SaveNewPark(*newPark)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	result.Success("新园区完成创建。")
}
